#define N64_PIN 2

#define N64_HIGH DDRD &= ~0x04
#define N64_LOW DDRD |= 0x04
#define N64_QUERY (PIND & 0x04)

unsigned char dataDump[32];
unsigned char sendData[4];


volatile unsigned char bitsSent;
char *sendByte;
char sendStep;

void N64_Send(char *data, int len);
char N64_Get(void);
char N64_Listen(void);
void print_binary(int v, int num_places);
void print_dataDump(void);
void clear_dataDump(void);
void br(void);

void N64_Send_OK( char *buffer, char length);
char N64_Get_OK(void);

void setup() {
  Serial.begin(115200);
  
  pinMode(N64_PIN, OUTPUT);
  digitalWrite(N64_PIN, LOW);
  N64_HIGH;
}

#define SEsND

void loop() {
  char command = 0x01;
  startTimer();
  N64_Send(&command, 1);
  endTimer();
  N64_Get();
  //print_dataDump();
}

/*
Set timer2 to 1000Hz
*/

void N64_Send(char *data, int len) {
  bitsSent = 0;
  sendByte = data;
  sendStep = 0;
  
  cli();
  TCCR2A = 0;
  TCCR2B = 0;
  TCNT2  = 0;
  
  OCR2A = 16;
  TCCR2B |= (1 << WGM12);
  TCCR2B |= 1 << CS10;
  TIMSK2 |= (1 << OCIE2A);
  sei();
  
  while (len * 8 > bitsSent) {}
  
  cli();
  TCCR2B = 0;
  TIMSK2 &= ~(1 << OCIE2A);
  sei();
  
  asm volatile ("nop\nnop\nnop\nnop\nnop\n"
                "nop\nnop\nnop\nnop\nnop\n");
  N64_HIGH;
}

ISR(TIMER2_COMPA_vect) {
  if (sendStep == 0) {
    N64_LOW;
  } else if (sendStep == 1 && *sendByte >> 7) {
    N64_HIGH;
  } else if (sendStep == 3) {
    N64_HIGH;
    
    ++bitsSent;
    *sendByte <<= 1;
    if (bitsSent % 8 == 0)
      ++sendByte;
  }
  
  sendStep = (sendStep + 1) % 4;
}

char N64_Get() {
  unsigned char *dump = dataDump;
  char bits = 0;
  int timeout;
  
  cli();
  while (bits != 32) {
    
    timeout = 64;
    while (N64_QUERY) {
      if (!--timeout) {
        sei();
        return bits;
      }
    }

    asm volatile("nop\nnop\nnop\nnop\nnop\n"
                 "nop\nnop\nnop\nnop\nnop\n"
                 "nop\nnop\nnop\nnop\nnop\n"
                 "nop\nnop\nnop\nnop\nnop\n"
                 "nop\nnop\nnop\nnop\nnop\n"
                 "nop\nnop\nnop\nnop\nnop\n");
    
    *dump = N64_QUERY;
    dump++;
    bits++;
    
    timeout = 64;
    while (!N64_QUERY) {
      if (!--timeout) {
        sei();
        return bits;
      }
    }
  }
  sei();
  return bits;
}

char N64_Listen() {
  int timeout = 0xff;
  while (N64_QUERY) {
    if (!--timeout)
      return 0;
  }
  return 1;
}

void print_binary(int v, int num_places)
{
    int mask=0, n;

    for (n=1; n<=num_places; n++)
    {
        mask = (mask << 1) | 0x0001;
    }
    v = v & mask;  // truncate v to specified number of places

    while(num_places)
    {

        if (v & (0x0001 << num_places-1))
        {
             Serial.print("1");
        }
        else
        {
             Serial.print("0");
        }

        --num_places;
        if(((num_places%4) == 0) && (num_places != 0))
        {
            Serial.print("_");
        }
    }
}

void print_dataDump() {
  for (int i = 0; i < 32; i++) {
    Serial.print((int)dataDump[i]);
  }
  br();
}

void clear_dataDump(void) {
  for (int i = 0; i < 32; i++) {
    dataDump[i] = 0;
  }
}

void br() {
  Serial.println();
}

//  Serial.setTimeout(50);
//  Serial.print((char)0x01);
//  Serial.readBytes(sendData, 4);
  
//  debug(
//  for (int i = 0; i < 5; i++) {
//    print_binary(sendData[i], 8);
//    Serial.print('_');
//  }
//  br();
//  )

/*
noInterrupts();
  char message = 0x01;
  N64_Send(&message, 1);
  N64_Get();
  interrupts();
  
  print_dataDump();
  delay(100); 
  */
